#include "ray.h"
#include "sphere.h"

Scalar Ray::intersect(Sphere s) {
    Vec3 v = origin - s.center;

    /* let's find a,b,c, such that:
     *    a*k^2 + b*k + c = 0
     * iff pos(k) is on the sphere.
     * That is, iff pos(k) is distant s.radius from s.center.
     * (solve on paper, report the solution below).
     */
    Scalar a = 1;
    Scalar b = 2*dot(v, direction);
    Scalar c = dot(v, v) - s.radius * s.radius;

    Scalar delta = b*b - 4*a*c;

    if (delta < 0) return MISS;

    Scalar k = -b - sqrt(delta) / (2*a);

    if (k < 0) return MISS;

    return k;
}
