#pragma once
#include <math.h>
#include"scalar.h"
struct Vec3;

Scalar dot(const Vec3 &a, const Vec3 &b);

Vec3 cross(const Vec3 &a, const Vec3 &b);



Scalar degrees_between(const Vec3 &a, const Vec3 &b);

struct Vec3 {    // Struct è sinonimo di class ma l'unica diff. è che nella struct le cose sono pubbliche di default (non devo mettere "public:" prima delle cose pubbliche)
    Scalar x;   // Scrivere "double x = 0" vorrebbe dire che in ogni costruttore la prima cosa fatta è questo assegnamento
    Scalar y;
    Scalar z;

    Vec3(Scalar _x, Scalar _y, Scalar _z) : x(_x), y(_y), z(_z) {}  // Questa è la sintassi per inizializzare i campi nel costruttore (è un'inizializzazione, che potrei fare anche sulle costanti [su cui assegnamenti non li posso fare])  
    
    // Avendo messo un costruttore ho tolto il costruttore di default (che non inizializzava i campi, allocava solo lo spazio -> il valore era undefined, il vecchio valore in memoria)
    Vec3() : Vec3(0,0,0) {} //In questo caso invochiamo l'altro costruttore

    void scale(Scalar k) {
        this->x *= k;          //Perché this è un puntatore accedo al suo membro x con la freccetta
        y *= k;
        z *= k;
    }

    Vec3 scaled(Scalar k) const {     // "scaled" perché mi restituisce la sua versione scalata; "const" qui indica che l'oggetto sui cui è stato chiamato non cambierà al compilatore
        return Vec3(x * k, y * k, z * k);
    }

    // Il passaggio di parametri è fatto sempre per copia, e gli oggetti pure vengono copiati.
    // Per passare per riferimento bisogna mettere & prima del nome del parametro (Vec3 &other): qui conviene farlo per risparmiare memoria!
    // Per assicurare al chiamante che anche se me lo faccio passare per riferimento non lo cambierò metto const davanti al nome -> il compilatore mi controlla!
    void add(const Vec3 &other) {
        x += other.x;
        y += other.y;
        z += other.z;
    }

    Vec3 added(const Vec3 &other) const {  
        return Vec3(x + other.x, y + other.y, z + other.z);
    }

    Scalar length() const {
        return std::sqrt(dot(*this,*this));
    }

    void normalize() {  
        scale(1 / length());     //*this = l'oggetto puntato da this, che è me stesso
    }

    Vec3 normalized() {
        return scaled(1 / length());
    }

    Vec3 operator *(Scalar k) const {     // Scrivendo operator * stiamo ridefinendo il * per i Vec3: però non posso inventare nuovi operatori, non posso cambiare il numero di parametri e neanche l'ordine di applicazione! (* sarà sempre prima di +)
        return Vec3(x * k, y * k, z * k);
    }

    void operator *=(Scalar k) {
        x *= k;          
        y *= k;
        z *= k;
    }

    Vec3 operator +(const Vec3 &other) const {  
        return Vec3(x + other.x, y + other.y, z + other.z);
    }

    void operator +=(const Vec3 &other) {
        x += other.x;
        y += other.y;
        z += other.z;
    }

    Vec3 operator -() const {
        return Vec3(-x,-y,-z);
    }

    Vec3 operator -(const Vec3 &other) const {
        return Vec3(x - other.x, y - other.y, z - other.z);
    }

    void operator -=(const Vec3 &other) {
        x -= other.x;
        y -= other.y;
        z -= other.z;
    }


    bool operator ==(const Vec3 &other) const {
        return x == other.x && y == other.y && z == other.z;    //Ordine operatori: unari -> binari, comparazione (==, <...) -> booleani
    }

    void printf() const;

    // friend dot -> Indico che la funz. globale dot può manipolare eventuali attributi privati di Vec3 
};

inline Vec3 operator *(Scalar k, const Vec3 &v){
    return v*k;
}
